// ---------------------------------------------------------------------------
//
//  This file is part of PermLib.
//
// Copyright (c) 2009-2010 Thomas Rehn <thomas@carmen76.de>
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
// 3. The name of the author may not be used to endorse or promote products
//    derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
// OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
// IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
// INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
// NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
// THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// ---------------------------------------------------------------------------


#define BOOST_TEST_DYN_LINK 
#define BOOST_TEST_MODULE BSGS generators test
#include <boost/test/unit_test.hpp>

#include <iostream>
#include <fstream>

#include "permutation.h"
#include "generator/product_replacement_generator.h"
#include "bsgs.h"
#include "construct/schreier_sims_construction.h"

#include "test-common.h"

#include <boost/foreach.hpp>


BOOST_AUTO_TEST_CASE( product_replacement_generator )
{
	typedef Permutation PERM;
	
    PERMlist generators;
	read(info_1997.filename, generators);
	
	boost::indirect_iterator<PERMlist::iterator> begin(generators.begin()), end(generators.end());
	std::vector<PERM> originalGenerators(begin, end);
	
	ProductReplacementGenerator<PERM> rng(generators.begin(), generators.end());
	for (uint i=0; i<100; ++i)
		rng.next();
	
	uint index = 0;
	BOOST_FOREACH(boost::shared_ptr<PERM> p, generators) {
		BOOST_CHECK( *p == originalGenerators[index] );
		++index;
	}
}
